#ifndef AUBO_SCOPE_MODBUS_IO_H
#define AUBO_SCOPE_MODBUS_IO_H

#include <aubo_caps/domain/io/io.h>

namespace arcs {
namespace aubo_scope {
ARCS_CLASS_FORWARD(ModbusIo);

/**
 *
 * This interface provides support for MODBUS I/Os.
 */
class ARCS_ABI_EXPORT ModbusIo : public Io
{
public:
    ModbusIo(ModbusIo &f);
    ModbusIo(ModbusIo &&f);
    ~ModbusIo();

    /**
     *
     * @return Get the IP address for this MODBUS I/O.
     */
    std::string getIpAddress();

    /**
     *
     * @return the signal address for this MODBUS I/O.
     */
    std::string getSignalAddress();

    /**
     *
     * @param value Set MODBUS signal to an integer value. If the MODBUS signal
     * is digital, then {@code false} is represented by 0 and {@code true} are
     * represented by all numbers different from 0.
     *
     * @return {@code true} if the MODBUS signal was sent down to the
     * controller, and {@code false} if the signal was not sent down, e.g.
     * because the controller was not running. Notice that the MODBUS signal
     * must support writes, consult the UR MODBUS protocol.
     */
    bool setValue(int value);

    /**
     *
     * @return the integer value of the MODBUS signal. If the MODBUS signal is
     * digital then {@code false} is represented by 0 and {@code true} is
     * represented by the value 1.
     */
    int getValue();

private:
    friend class DataSwitch;
    ModbusIo();
    void *d_{ nullptr };
};

} // namespace aubo_scope
} // namespace arcs

#endif // MODBUSIO_H
