#ifndef AUBO_SCOPE_KEYBOARD_INPUT_CALLBACK_H
#define AUBO_SCOPE_KEYBOARD_INPUT_CALLBACK_H

namespace arcs {
namespace aubo_scope {

/**
 * Callback used when the virtual keyboard/keypad is exited by the end user.
 * Override this abstract class to execute custom code when the end user accepts
 * or cancels the input.
 *
 * @param <T> The (generic) type parameter for the interface representing the
 * type of input data entered by the end user, e.g. {@link Integer} and {@link
 * Double}.
 */
template <typename T>
struct KeyboardInputCallback
{
    /**
     * This method is called if the end user accepts the keyboard/keypad input.
     * Overriding this method is mandatory.
     *
     * @param value accepted value.
     */
    virtual void onOk(T value) = 0;

    /**
     * This method is called if the end user cancels the input. Overriding this
     * method is optional.
     */
    virtual void onCancel() {}
};

} // namespace aubo_scope
} // namespace arcs

#endif
